VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAnalogWave"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'   clsAnalogWave
'
'
'   This class manages the data input/output.
'
'   Info about the pulse is stored (burst rate, etc.) here.
'   Also stores current gains per channel.
'
'
'   Important child classes:
'
'   clsAnalogWaveOutput     Data send buffers
'   clsAnalogWaveInput      Data read buffers
'
'
'   The Pulse_1, Pulse_2, and Pulse_3 functions manage the direct data I/O
'   with the box.  See function headers for more info.
'
'   The Initialize function resets the burst rate and scanlists.  This is
'   necessary before & after referencing, and if any pulse parameters change.
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit


Private mobjParent As clsMainDAC                        ' Pointer to parent class
Private mcolAnalogWaveInput As Collection               ' Array of wave in classes
Private mobjAnalogWaveOutput As clsAnalogWaveOutput     ' Wave out classes
Private mdblTotalDuration As Double                     ' Total duration of main pulse (ms)
Private mlngNumSamples As Long                          ' # of sample points in main pulse
Private mlngNumRefSamples As Long                       ' # of sample points in reference pulse
Private mblnDifferential As Boolean                     ' If using differential mode
Private mblnUnipolar As Boolean                         ' If using unipolar mode
Private mdblBaseTimeUnit As Double                      ' Effective sampling rate
Private mudtOutList As di_outlist_struct                ' Dataq's output scanlist
Private marrInList() As di_inlist_struct                ' Dataq's input scanlist
Private mintInputListLength As Integer                  ' Number of channels to use (input)
Private mintOutputListLength As Integer                 ' Number of channels to use (output)
Private mlngInputBufferSize As Long                     ' Input buffer size (bytes)
Private mlngOutputBufferSize  As Long                   ' Output buffer size
Private mlngInputBufferPtr As Long                      ' Returned by di_buffer_alloc
Private mlngOutputBufferPtr As Long                     ' Returned by di_buffer_alloc
Private mintBurstRate As Long                           ' Burst rate (Hz)
Private mlngScanRate As Long                            ' Scan rate (per channel)
Private mlngCounter As Long                             ' Points to average on DSP
Private mintDebugPoint As Integer                       ' For debugging
Private mblnScanning As Boolean                         ' While scanning
Private marrGains(1 To MAX_CHANNELS)                    ' Gain array
Private mblnAutoGainVoltage As Boolean                  ' Flag for auto gain
Private mblnAutoGainCurrent As Boolean                  ' Flag for auto gain

'Event PulseCompleted()                                  ' Not used


Public Property Get Gain(Index As Integer) As Integer
    If Index >= 1 And Index <= MAX_CHANNELS Then
        Gain = marrGains(Index)
    Else
        Gain = 0
    End If
End Property

Public Property Let Gain(Index As Integer, Value As Integer)
    If Index >= 1 And Index <= MAX_CHANNELS Then
        Select Case Value
            Case 1, 2, 4, 5, 8, 10, 100, 1000
                marrGains(Index) = Value

            Case Else
                ' Ignore
        End Select
    End If
End Property

Public Property Get AutoGainVoltage() As Boolean
    AutoGainVoltage = mblnAutoGainVoltage
End Property

Public Property Let AutoGainVoltage(Value As Boolean)
    mblnAutoGainVoltage = Value
End Property

Public Property Get AutoGainCurrent() As Boolean
    AutoGainCurrent = mblnAutoGainCurrent
End Property

Public Property Let AutoGainCurrent(Value As Boolean)
    mblnAutoGainCurrent = Value
End Property

Public Property Get TotalDuration() As Double
    TotalDuration = mdblTotalDuration
End Property

'Public Property Let TotalDuration(Value As Double)
'    mdblTotalDuration = Value
'    ' Update the BaseTimeUnit
'    If mlngNumSamples > 0 Then
'        mdblBaseTimeUnit = mdblTotalDuration / mlngNumSamples
'    Else
'        mdblBaseTimeUnit = 0
'    End If
'End Property

Public Property Get NumSamples() As Long
    NumSamples = mlngNumSamples
End Property

'Public Property Let NumSamples(Value As Long)
'    Dim objAnaIn As clsAnalogWaveInput
'    mlngNumSamples = Value
'    ' Update the BaseTimeUnit
'    If mlngNumSamples > 0 Then
'        mdblBaseTimeUnit = mdblTotalDuration / mlngNumSamples
'    Else
'        mdblBaseTimeUnit = 0
'    End If
'    ' Resize the children's arrays to fit this new size.
'    mobjAnalogWaveOutput.NumSamples = Value
'    For Each objAnaIn In mcolAnalogWaveInput
'        objAnaIn.NumSamples = Value
'    Next objAnaIn
'End Property

'Public Property Let NumRefSamples(Value As Long)
'    mlngNumRefSamples = Value
'End Property

Public Property Get BaseTimeUnit() As Double
    BaseTimeUnit = mdblBaseTimeUnit
End Property

Public Property Get Differential() As Boolean
    Differential = mblnDifferential
End Property

Public Property Let Differential(Value As Boolean)
    mblnDifferential = Value
End Property

Public Property Get Unipolar() As Boolean
    Unipolar = mblnUnipolar
End Property

Public Property Let Unipolar(Value As Boolean)
    mblnUnipolar = Value
End Property

Public Property Let Parent(objParent As clsMainDAC)
    Set mobjParent = objParent
End Property

Public Property Get Parent() As clsMainDAC
    Set Parent = mobjParent
End Property

Public Property Get AnalogWaveIn(Channel As Integer) As clsAnalogWaveInput
    If Channel >= 1 And Channel <= mcolAnalogWaveInput.Count Then
        Set AnalogWaveIn = mcolAnalogWaveInput(Channel)
    Else
        Set AnalogWaveIn = Nothing
    End If
End Property

Public Property Get AnalogWaveOut() As clsAnalogWaveOutput
    Set AnalogWaveOut = mobjAnalogWaveOutput
End Property

'
' Specify the Burst rate and the Count weight
' and allocate buffers.
'
' --------------------------------------------------
'         B                        B                         S = Desired Sampling Rate of the Input List
'  S = ---------     <=>     C = ------ - 1          Where:  B = Burst Rate of the DSP
'      L (C + 1)                 S * L                       L = Greater Length of either Input or Output List
'                                                            C = Counter Weight to be used in Input List
Public Sub Initialize(blnReference As Boolean)
    Dim lngIndex As Long
    Dim intTissue As Integer
    Dim lngSample As Long
    Dim lngRetVal As Long
    Dim intInputListIndex As Integer
    Dim intBurstRateCount As Integer


    ' Original Gaizka code:
    ' Find Out how big the Input List is...2 entries for each active tissue
    'mintInputListLength = 0
    'For intTissue = 1 To MAX_TISSUES
    '    If mobjParent.Tissue(intTissue) Then
    '        mintInputListLength = mintInputListLength + 2
    '    End If
    'Next intTissue

    ' (ED) 4-28-06:  Set to the same length every time to avoid crosstalk issues
    mintInputListLength = 2 * MAX_TISSUES


    If mintInputListLength > 0 Then
        ReDim Preserve marrInList(0 To mintInputListLength - 1)


        ' This would be the maximum possible burst rate
        'mintBurstRate = mobjParent.DAC_Device.ClockFrequency / mobjParent.DAC_Device.MinBurstRateCount     ' Assume Slowest burst rate

        ' Set below safe levels to avoid crosstalk noise
        mintBurstRate = 180000

        ' Figure burst rate count divisor (to be set later in di_burstrate)
        intBurstRateCount = mobjParent.DAC_Device.ClockFrequency / mintBurstRate

        ' Per channel sampling rate on DSP
        mlngScanRate = mintBurstRate / mintInputListLength

        ' For example:  Assuming 34 samples, a scan rate of 10000, and 620 ms
        ' total pulse length, the counter is 182.

        ' Calculate # of points to sample on DSP
        mlngCounter = mlngScanRate / mlngNumSamples * (mdblTotalDuration / 1000)

        ' This effectively sets the actual sampling rate per point
        ' which is:  mdblTotalDuration / mlngNumSamples
        ' If duration is 620, and we have 34 points then this is 18.2 msec


        'If mlngCounter < 10 Then
        '    ' We have a potential problem is the counter is too small -> no averaging...
        '    Debug.Print "Initialize COUNTER VALUE TOO SMALL !"
        'End If
    End If


    ' ED 5-11-06: Originally Gaizka was turning off channels that
    ' were not selected in the scanlist.  I believe the intention
    ' was originally to cut down on datafile size by not recording
    ' values for tissues that were not active.  However, that means
    ' that the sampling rate will change depending on how many tissues
    ' are turned on.

    ' If we always keep the same scanlist regardless of what channels
    ' are physically active, the DSP will keep a steady rate when it
    ' computes the average.

    ' So we are fixing the number of tissues/channels to the max value
    ' and keeping all turned on.  The code that reads back the values
    ' from the DI array is in Pulse_3.


    ' Set the inlist entries
    intInputListIndex = 0
    For intTissue = 1 To MAX_TISSUES

        ' ED: Commented out
        ' Gaizka:  Test to see if the channel is in the input list
        'If mobjParent.Tissue(intTissue) Then
            ' Add the channel to the input list (First Bank)

            marrInList(intInputListIndex).Ave = 1                      ' Use Averaging
            marrInList(intInputListIndex).chan = intTissue - 1         ' Select channel
            marrInList(intInputListIndex).counter = mlngCounter        ' Counter is really the number of samples to average
            marrInList(intInputListIndex).diff = 0                     ' No differential
            marrInList(intInputListIndex).dig_out = 0                  ' No digital
            marrInList(intInputListIndex).dig_out_enable = 0           ' No digital
            marrInList(intInputListIndex).Gain = ConvertToDACGain(Gain(intTissue))
            marrInList(intInputListIndex).Unipolar = 0                 ' Bipolar

            intInputListIndex = intInputListIndex + 1

            ' Add the channel to the input list (Second bank)
            marrInList(intInputListIndex).Ave = 1                     ' Use Averaging
            marrInList(intInputListIndex).chan = intTissue + MAX_TISSUES - 1         ' Select channel
            marrInList(intInputListIndex).counter = mlngCounter        ' Counter is really the number of samples to average
            marrInList(intInputListIndex).diff = 0                     ' No differential
            marrInList(intInputListIndex).dig_out = 0                  ' No digital
            marrInList(intInputListIndex).dig_out_enable = 0           ' No digital
            marrInList(intInputListIndex).Gain = ConvertToDACGain(Gain(intTissue + MAX_TISSUES))
            marrInList(intInputListIndex).Unipolar = 0                ' Bipolar

            intInputListIndex = intInputListIndex + 1

        ' ED: Commented out
        'End If

    Next intTissue


    ' Set the only outlist entry item
    mudtOutList.counter = mlngCounter           ' IMPORTANT: only 1 sample from the output for each ENTIRE LIST of the input
    mudtOutList.dig_data = 0
    mudtOutList.dig_out_enable = 0
    mudtOutList.digital = 0
    mudtOutList.Unipolar = 0
    mintOutputListLength = 1

    ' Why is DACPort set to 0 (instead of 1) for DI-720?
    'mobjParent.DAC_Device.DACPort


    ' Clear the Input(0) and Output(1) buffers
    Call di_buffer_free(0)
    Call di_buffer_free(1)

    ' Allocate the Input and Output buffers
    mlngInputBufferSize = mintInputListLength * mlngNumSamples * 2
    mlngOutputBufferSize = mlngNumSamples

    ' Check to enable the minimum buffer size required
    If mlngInputBufferSize > 0 Then
        ' We have an Input buffer to allocate
        If mlngInputBufferSize < mobjParent.DAC_Device.MinInputBufferSize Then
            mlngInputBufferSize = mobjParent.DAC_Device.MinInputBufferSize
        End If

        ' Allocate the Input buffer
        mlngInputBufferPtr = di_buffer_alloc(0, mlngInputBufferSize)
        'Debug.Print "Initialize [di_buffer_alloc (input, " & mlngInputBufferSize & ")] = " & mlngInputBufferPtr

        If mlngOutputBufferSize < mobjParent.DAC_Device.MinOutputBufferSize Then
            mlngOutputBufferSize = mobjParent.DAC_Device.MinOutputBufferSize
        End If
    Else
        ' We only have one buffer to allocate (the output buffer)
        If mlngOutputBufferSize < mobjParent.DAC_Device.MinSingleBufferSize Then
            mlngOutputBufferSize = mobjParent.DAC_Device.MinSingleBufferSize
        End If
    End If

    ' Allocate the Output Buffer
    mlngOutputBufferPtr = di_buffer_alloc(1, mlngOutputBufferSize)
    'Debug.Print "Initialize [di_buffer_alloc (output, " & mlngOutputBufferSize & ")] = " & mlngOutputBufferPtr

    lngRetVal = di_list_length(mintInputListLength, mintOutputListLength)
    'Debug.Print "Initialize [di_list_length] = " & lngRetVal
    lngRetVal = di_outlist(mudtOutList)
    'Debug.Print "Initialize [di_outlist] = " & lngRetVal

    If mintInputListLength > 0 Then
        lngRetVal = di_inlist(marrInList(0))
        'Debug.Print "Initialize [di_inlist] = " & lngRetVal
    End If

    'Debug.Print "Initialize [(In/Out)List Counter = " & mlngCounter & "] "

    'lngRetVal = di_burst_rate(mobjParent.DAC_Device.MinBurstRateCount)
    lngRetVal = di_burst_rate(intBurstRateCount)

    'Debug.Print "Initialize [di_burst_rate(" & mobjParent.DAC_Device.MinBurstRateCount & ")] = " & lngRetVal & " at BTU: " & mdblBaseTimeUnit & " ms/sample"

'    If mlngScanRate > mobjParent.DAC_Device.MinBurstRateCount And mlngScanRate < mobjParent.DAC_Device.MaxBurstRateCount Then
'        lngRetVal = di_burst_rate(mlngScanRate)
'        Debug.Print "Initialize [di_burst_rate(" & mlngScanRate & ")] = " & lngRetVal & " at BTU: " & mdblBaseTimeUnit & " ms/sample"
'    Else
'        Debug.Print "Invalid Burst Rate = " & mintBurstRate
'        'MsgBox "Invalid Burst Rate: " & mintBurstRate, vbExclamation
'    End If
End Sub

Public Sub AssignWave()
    ' Copy the output buffer to the DAC
    Call mobjAnalogWaveOutput.CopyWaveToDAC
End Sub

Public Sub BuildFlatWave()
    Dim lngSample As Long


    ' Proceed to build the output signal
    For lngSample = 1 To mlngNumRefSamples
    'For lngSample = 1 To mlngNumSamples
        mobjAnalogWaveOutput.ItemVolts(lngSample) = 0
    Next lngSample
End Sub

' Here's where we are making the pulse
Public Sub BuildDefaultWave()
    Dim intStep As Integer
    Dim lngSample As Long


    ' Proceed to build the output signal

    ' Split into 4 parts
    intStep = mlngNumSamples / 4

    ' First part hold at zero (will be Isc)
    For lngSample = 1 To intStep
        'mobjAnalogWaveOutput.ItemBits(lngSample) = 0
        mobjAnalogWaveOutput.ItemVolts(lngSample) = 0
    Next lngSample

    ' Second part (positive pulse)
    For lngSample = intStep To 2 * intStep
        'mobjAnalogWaveOutput.ItemBits(lngSample) = &H4096
        mobjAnalogWaveOutput.ItemVolts(lngSample) = 10       ' 5 mV
    Next lngSample

    ' Third part hold at zero
    For lngSample = 2 * intStep To 3 * intStep
        'mobjAnalogWaveOutput.ItemBits(lngSample) = 0
        mobjAnalogWaveOutput.ItemVolts(lngSample) = 0
    Next lngSample

    ' Fourth part (negative pulse)
    For lngSample = 3 * intStep To mlngNumSamples
        'mobjAnalogWaveOutput.ItemBits(lngSample) = &H8000
        mobjAnalogWaveOutput.ItemVolts(lngSample) = -10      ' -5 mV
    Next lngSample

    'mobjAnalogWaveOutput.ItemBits(mlngNumSamples) = 0
    mobjAnalogWaveOutput.ItemVolts(lngSample) = 0
End Sub

Public Sub StopScan()
    Dim lngRetVal As Long

    mblnScanning = False
    lngRetVal = di_stop_scan

    'Debug.Print "Stop Scan = " & lngRetVal
End Sub

'------------------------------------------------------------------------------------------------------------------
'
'   Pulse_1
'
'
'   This function takes no inputs, but a valid scanlist must have been already created.
'
'   This function sends the command to the data acquisition hardware to carry out a new scan.
'
'   The function terminates after ensuring the scan is sent successfully.
'
'   The calling program is then responsible for checking the results of the scan elsewhere.
'
'------------------------------------------------------------------------------------------------------------------
Public Sub Pulse_1()
    Dim lngRetVal As Long
    'Dim objAnalogWaveIn As clsAnalogWaveInput
    'Dim intTissue As Integer
    'Dim intNumTissues As Integer
    Dim lngBufferStatus0 As Long
    Dim lngBufferStatus1 As Long
    Dim lngMaxBufferValue As Long

    'Dim lngSample As Long
    'Dim intWaveNum As Integer
    'Dim arrWave() As Integer
    'Dim dblAverageValue As Double
    'Dim lngMinValue As Long
    'Dim lngMaxValue As Long
    'Dim sglTimer As Single
    'Dim intChannelIndex As Integer

    'On Error GoTo EH


    'Debug.Print "Pulse_1"

    mblnScanning = True
    mintDebugPoint = 0


    ' Calculate how many active tissues there are...
    'intNumTissues = 0
    'For intTissue = 1 To MAX_TISSUES
    '    intNumTissues = intNumTissues + IIf(mobjParent.Tissue(intTissue), 1, 0)
    'Next intTissue


    'While mblnScanning
        'sglTimer = Timer
        lngRetVal = di_start_scan()
        
        'Debug.Print "Input/Output Buffer Status: " & di_buffer_status(0), di_buffer_status(1)
        
        'DoEvents

        mintDebugPoint = 1
        
        Do
            lngBufferStatus0 = di_buffer_status(0)
            lngBufferStatus1 = di_buffer_status(1)

            'Debug.Print "Input/Output Buffer Status: " & lngBufferStatus0, lngBufferStatus1

            If lngBufferStatus0 > lngMaxBufferValue Then
                lngMaxBufferValue = lngBufferStatus0
            End If

            If lngBufferStatus0 < lngMaxBufferValue Then
                ' Oops, looping
                Exit Do
            End If

            'DoEvents
        Loop While lngBufferStatus1 > mlngNumSamples  '0 '(di_buffer_status(1)) > 0 'mlngNumSamples

        mintDebugPoint = 2


    Exit Sub

EH:
    'Debug.Print "Error in clsAnalogWave::Pulse"
    'Call MsgBox("Error at point " & mintDebugPoint & "  " & Err.Description)
    Call RecordError("clsAnalogWave::Pulse_1 #" & Err.Number & ": " & Err.Description)
End Sub

'------------------------------------------------------------------------------------------------------------------
'
'   Pulse_2
'
'
'   This function takes no inputs, but a valid scan should be in progress.
'
'   This function checks the data acquisition hardware to see if the data is ready.
'
'   The function returns immediately with true (data not ready) or false (data is ready).
'   In case that seems intuitively backwards think about it from the perspective of asking
'   the question in the calling program "Do we need to keep waiting?"  The answer is then Yes
'   or True if there is no data ready, and No if we do not need to wait any longer.
'
'------------------------------------------------------------------------------------------------------------------
Public Function Pulse_2() As Boolean
    Dim blnLoop As Boolean

    'On Error GoTo EH


    'While (di_buffer_status(1) < mlngNumSamples + 1) And mblnScanning '* mintInputListLength
        'DoEvents
        'Debug.Print "Input/Output Buffer Status: " & di_buffer_status(0), di_buffer_status(1)
    'Wend

    'Debug.Print "Pulse_2"

    blnLoop = (di_buffer_status(1) < mlngNumSamples + 1) And mblnScanning

    '* mintInputListLength
        
    mintDebugPoint = 3

    'Debug.Print "Pulse [StartScan] = " & lngRetVal

    If Not blnLoop Then
        Call di_stop_scan
    End If


    ' True if we need to wait
    ' False if we are done
    Pulse_2 = blnLoop


    'sglTimer = Timer - sglTimer
    'Debug.Print "Input/Output Buffer Status: " & di_buffer_status(0), di_buffer_status(1)
    'DoEvents
    'Debug.Print "Pulse Duration: " & Format(sglTimer, "0.0000") & " seconds"
    'sglTimer = Timer

    'sglTimer = Timer - sglTimer
    'Debug.Print "Pulse Processing: " & Format(sglTimer, "0.0000") & " seconds"
    'mblnScanning = False
    'Wend

    'mintDebugPoint = 6


    Exit Function

EH:
    'Debug.Print "Error in clsAnalogWave::Pulse_2 " & Err.Description
    'Call MsgBox("Error at point " & mintDebugPoint & "  " & Err.Description)
    Call RecordError("clsAnalogWave::Pulse_2 #" & Err.Number & ": " & Err.Description)
End Function

'------------------------------------------------------------------------------------------------------------------
'
'   Pulse_3
'
'
'   This function takes no inputs, but a valid scan should have just completed.
'
'   This function reads the returned scan list and stores in a local buffer.
'
'   The function returns after storing the data.
'
'------------------------------------------------------------------------------------------------------------------
Public Sub Pulse_3()
    Dim lngRetVal As Long
    Dim objAnalogWaveIn As clsAnalogWaveInput
    Dim intTissue As Integer
    Dim intNumTissues As Integer
    Dim intSample As Integer
    Dim arrWave() As Integer
    Dim intSignIntBitVal As Integer
    Dim lngItemBits As Long
    Dim intArrayIndex As Integer

    'Dim intWaveNum As Integer
    'Dim dblAverageValue As Double
    'Dim lngMinValue As Long
    'Dim lngMaxValue As Long
    'Dim sglTimer As Single
    'Dim intChannelIndex As Integer
    'Dim lngMaxBufferValue As Long
    'Dim lngBufferStatus0 As Long
    'Dim lngBufferStatus1 As Long

    ' Calling function handles error
    'On Error GoTo EH


    mblnScanning = False
    mintDebugPoint = 4


    ' Gaizka:  Calculate how many active tissues there are
    'intNumTissues = 0
    'For intTissue = 1 To MAX_TISSUES
    '    intNumTissues = intNumTissues + IIf(mobjParent.Tissue(intTissue), 1, 0)
    'Next intTissue


    ' ED:  Fix # of tissues
    ' See comments in Initialize
    intNumTissues = MAX_TISSUES


    ' Move the data into the arrays
    If mlngInputBufferSize > 0 Then
        ReDim arrWave(0 To mlngInputBufferSize - 1)
        lngRetVal = di_copy_buffer(0, arrWave(0), mlngInputBufferSize)
        'Debug.Print "di_copy_buffer(0, arrWave(0), " & mlngInputBufferSize & ") =" & lngRetVal
    End If


    ' Looping through each tissue and then each sample minimizes the # of times
    ' the Set objAnalogWaveIn is called which speeds code slightly.

    ' Data is stored in array by sample first, then tissue, then bank offset (volt/curr)


    ' Copy the DI array (arrWave) to the class object (mcolAnalogWaveInput)


    ' Tracks active tissue number
    'intChannelIndex = 0
    For intTissue = 1 To MAX_TISSUES
        ' mobjParent.Tissue stores whether tissue is on or off
        If mobjParent.Tissue(intTissue) Then
            ' ED:  This should now be incremented *every* step (use intTissue instead)
            'intChannelIndex = intChannelIndex + 1

            ' Work with the first bank (Voltages)
            Set objAnalogWaveIn = mcolAnalogWaveInput(intTissue)

            ' Zero-based index to reference DI-720's wave array
            ' mlngNumSamples should be a small number, like 10, easily below 65536 integer cutoff
            For intSample = 0 To mlngNumSamples - 1
                'create the AnalogWaveIn array from 1 to NumSamples (10+10+X+10) where X comes from pulse interval
                'objAnalogWaveIn.ItemBits(lngSample + 1) = arrWave(lngSample * 2 * intNumTissues + (intChannelIndex - 1) * 2)

                ' Array index value
                'intArrayIndex = lngSample * 2 * intNumTissues + (intChannelIndex - 1) * 2
                intArrayIndex = GetArrayIndex(intNumTissues, intSample, intTissue, True)

                ' Get bit value
                intSignIntBitVal = arrWave(intArrayIndex)

                ' One-based index for our bit array
                Call objAnalogWaveIn.SetItemBits(intSample + 1, intSignIntBitVal)
            Next intSample


            ' Work with the second bank (Currents)
            Set objAnalogWaveIn = mcolAnalogWaveInput(intTissue + MAX_TISSUES)

            ' Zero-based index to reference DI-720's wave array
            For intSample = 0 To mlngNumSamples - 1
                'objAnalogWaveIn.ItemBits(lngSample + 1) = arrWave(lngSample * 2 * intNumTissues + (intChannelIndex - 1) * 2 + 1)

                ' Array index value
                'intArrayIndex = lngSample * 2 * intNumTissues + (intChannelIndex - 1) * 2 + 1
                intArrayIndex = GetArrayIndex(intNumTissues, intSample, intTissue, False)

                ' Get bit value
                intSignIntBitVal = arrWave(intArrayIndex)

                ' One-based index for our bit array
                Call objAnalogWaveIn.SetItemBits(intSample + 1, intSignIntBitVal)
            Next intSample
        End If
    Next intTissue


    mintDebugPoint = 5

    'sglTimer = Timer - sglTimer
    'Debug.Print "Pulse Processing: " & Format(sglTimer, "0.0000") & " seconds"
    'mblnScanning = False
    'Wend
    'intDebugPoint = 6


    Exit Sub

EH:
    'Debug.Print "Error in clsAnalogWave::Pulse_3"
    'Call MsgBox("Error at point " & mintDebugPoint & "  " & Err.Description)
    Call RecordError("clsAnalogWave::Pulse_3 #" & Err.Number & ": " & Err.Description)
End Sub

' Data is stored in array by sample first, then tissue, then bank offset (volt/curr)
Private Function GetArrayIndex(intNumTissues As Integer, intSample As Integer, intTissue As Integer, blnVoltBank As Boolean) As Integer
    Dim intBank As Integer


    ' intSample is zero-based
    ' intTissue is one-based
    ' intNumTissues is max # of tissues (i.e. 8)
    ' blnVoltBank is true for voltage and false for current

    ' intBank is 1 for voltage and 2 for current
    If blnVoltBank Then
        intBank = 1
    Else
        intBank = 2
    End If


    GetArrayIndex = intSample * 2 * intNumTissues + (intTissue - 1) * 2 + (intBank - 1)
End Function

'------------------------------------------------------------------------------------------------------------------
'
'   Pulse
'
'
'   The original pulsing routine.
'
'------------------------------------------------------------------------------------------------------------------
Public Sub Pulse()
    Dim lngRetVal As Long
    Dim objAnalogWaveIn As clsAnalogWaveInput
    Dim intTissue As Integer
    Dim intNumTissues As Integer
    Dim lngSample As Long
    Dim intWaveNum As Integer
    Dim arrWave() As Integer
    Dim dblAverageValue As Double
    Dim lngMinValue As Long
    Dim lngMaxValue As Long
    Dim sglTimer As Single
    Dim intChannelIndex As Integer
    Dim lngMaxBufferValue As Long
    Dim lngBufferStatus0 As Long
    Dim lngBufferStatus1 As Long


    mblnScanning = True
    
    ' Calculate how many active tissues there are...
    intNumTissues = 0
    For intTissue = 1 To MAX_TISSUES
        intNumTissues = intNumTissues + IIf(mobjParent.Tissue(intTissue), 1, 0)
    Next intTissue
    
    While mblnScanning
        sglTimer = Timer

        lngRetVal = di_start_scan()
        
        'Debug.Print "Input/Output Buffer Status: " & di_buffer_status(0), di_buffer_status(1)
        
        DoEvents
        
        Do
            lngBufferStatus0 = di_buffer_status(0)
            lngBufferStatus1 = di_buffer_status(1)
            'Debug.Print "Input/Output Buffer Status: " & lngBufferStatus0, lngBufferStatus1
            If lngBufferStatus0 > lngMaxBufferValue Then
                lngMaxBufferValue = lngBufferStatus0
            End If
            If lngBufferStatus0 < lngMaxBufferValue Then
                ' Oops, looping
                Exit Do
            End If
            DoEvents
        Loop While lngBufferStatus1 > mlngNumSamples  '0 '(di_buffer_status(1)) > 0 'mlngNumSamples

        'Call MsgBox("Before loop")

        While (di_buffer_status(1) < mlngNumSamples + 1) And mblnScanning '* mintInputListLength
            DoEvents
            'Debug.Print "Input/Output Buffer Status: " & di_buffer_status(0), di_buffer_status(1)
        Wend

        'Call MsgBox("After loop")

        'Debug.Print "Pulse [StartScan] = " & lngRetVal
        Call di_stop_scan
        sglTimer = Timer - sglTimer
        'Debug.Print "Input/Output Buffer Status: " & di_buffer_status(0), di_buffer_status(1)
        DoEvents
        'Debug.Print "Pulse Duration: " & Format(sglTimer, "0.0000") & " seconds"
        sglTimer = Timer
        
        ' Move the data into the arrays
        If mlngInputBufferSize > 0 Then
            ReDim arrWave(0 To mlngInputBufferSize - 1)
            lngRetVal = di_copy_buffer(0, arrWave(0), mlngInputBufferSize)
            'Debug.Print "di_copy_buffer(0, arrWave(0), " & mlngInputBufferSize & ") =" & lngRetVal
        End If
        
        intChannelIndex = 0
        For intTissue = 1 To MAX_TISSUES
            If mobjParent.Tissue(intTissue) Then
                intChannelIndex = intChannelIndex + 1
                
                ' Work with the first bank...
                Set objAnalogWaveIn = mcolAnalogWaveInput(intTissue)
                ' Copy the array to the object
                For lngSample = 0 To mlngNumSamples - 1
                    objAnalogWaveIn.ItemBits(lngSample + 1) = arrWave(lngSample * 2 * intNumTissues + (intChannelIndex - 1) * 2)
                    'Debug.Print "AnalogWaveIn[" & lngSample + 1 & "] = " & objAnalogWaveIn.ItemBits(lngSample + 1)
                Next lngSample
            
                ' Work with the second bank...
                Set objAnalogWaveIn = mcolAnalogWaveInput(intTissue + MAX_TISSUES)
                ' Copy the array to the object
                For lngSample = 0 To mlngNumSamples - 1
                    objAnalogWaveIn.ItemBits(lngSample + 1) = arrWave(lngSample * 2 * intNumTissues + (intChannelIndex - 1) * 2 + 1)
                    'Debug.Print "AnalogWaveIn[" & lngSample + 1 & "] = " & objAnalogWaveIn.ItemBits(lngSample + 1)
                Next lngSample
            End If
        Next intTissue
        
        sglTimer = Timer - sglTimer
        'Debug.Print "Pulse Processing: " & Format(sglTimer, "0.0000") & " seconds"
        mblnScanning = False
    Wend
End Sub

Public Sub ReleaseParent()
    Dim objAnalogWaveInput As clsAnalogWaveInput


    Call mobjAnalogWaveOutput.ReleaseParent
    For Each objAnalogWaveInput In mcolAnalogWaveInput
        objAnalogWaveInput.ReleaseParent
    Next objAnalogWaveInput

    Set objAnalogWaveInput = Nothing
    Set mobjParent = Nothing
End Sub

Public Sub SetDuration(dblPulseWidth As Double, lngSamples As Long, lngRefSamples As Long)
    Dim objAnaIn As clsAnalogWaveInput


    ' Pulse width (msec)
    mdblTotalDuration = dblPulseWidth

    ' Samples per pulse, for main pulse
    mlngNumSamples = lngSamples

    ' For reference pulse
    mlngNumRefSamples = lngRefSamples


    ' Update the BaseTimeUnit (effective sample rate)
    If mlngNumSamples > 0 Then
        mdblBaseTimeUnit = mdblTotalDuration / mlngNumSamples
    Else
        mdblBaseTimeUnit = 0
    End If
    

    ' Resize the children's arrays to fit this new size.
    mobjAnalogWaveOutput.NumSamples = mlngNumSamples

    For Each objAnaIn In mcolAnalogWaveInput
        objAnaIn.NumSamples = mlngNumSamples
    Next objAnaIn
End Sub

Private Sub Class_Initialize()
    Dim intChan As Integer
    Dim objAnalogWaveInput As clsAnalogWaveInput


    Set mcolAnalogWaveInput = New Collection
    Set mobjAnalogWaveOutput = New clsAnalogWaveOutput

    mobjAnalogWaveOutput.Parent = Me

    For intChan = 1 To MAX_CHANNELS
        Set objAnalogWaveInput = New clsAnalogWaveInput

        objAnalogWaveInput.Channel = intChan                ' Initialize the object with the channel number
        mcolAnalogWaveInput.Add objAnalogWaveInput
        objAnalogWaveInput.Parent = Me
    Next intChan

    Set mobjParent = Nothing

    mblnAutoGainVoltage = False
    mblnAutoGainCurrent = False

    DACAddReferenceCount "clsAnalogWave"
End Sub

Private Function ConvertToDACGain(Value As Integer)
    Dim intRetVal As Integer
    
    Select Case Value
        Case 1
            intRetVal = 0
        Case 2
            intRetVal = 1
        Case 4
            intRetVal = 2
        Case 5
            intRetVal = 2
        Case 8
            intRetVal = 3
        Case 10
            intRetVal = 3
        'Case 100
        '    intRetVal = 2
        'Case 1000
        '    intRetVal = 3
    End Select
    
    ConvertToDACGain = intRetVal
End Function

Private Sub Class_Terminate()
    Erase marrInList
    Erase marrGains

    DACRemoveReferenceCount "clsAnalogWave"
End Sub
