VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsReference"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'   clsReference
'
'
'   This class represents the reference object.  It stores the reference
'   values per tissue and per gain.
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit


' Store reference per channel per gain value
Private marrReference(1 To MAX_TISSUES, 1 To NUM_GAINS) As ReferenceValue
Private marrRefSatCurr(1 To MAX_TISSUES, 1 To NUM_GAINS) As Boolean
Private marrRefSatVolt(1 To MAX_TISSUES, 1 To NUM_GAINS) As Boolean
Private mobjParent As clsMainDAC
Private mblnReferenced As Boolean
'Private mblnApplyReference As Boolean
'Private mblnSaturatedValues As Boolean


Public Property Let Referenced(Value As Boolean)
    mblnReferenced = Value
End Property

Public Property Get Referenced() As Boolean
    Referenced = mblnReferenced
End Property

'**********************************************************************
' ClearReference
' This method is used to reset the reference and force a reference from
' the application
'**********************************************************************
Public Sub ClearReference()
    mblnReferenced = False
End Sub

' Old code (doesn't get used anymore)
Public Function Reference() As Long
    Dim lngRetVal As Long
    Dim intTissue As Integer
    Dim intSample As Integer
    Dim intNumSamples As Integer
    Dim dblValue As Double
    Dim lngValue As Long


    If mobjParent.IsOpen Then
        intNumSamples = 100

        ' Output a zero analog voltage
        mobjParent.DAC_AnalogOutput.DataVolts = 0
        mobjParent.DAC_AnalogOutput.Range = mobjParent.DAC_Device.Range                     ' Specifies the correct polarity
        mobjParent.DAC_AnalogOutput.DACPort = mobjParent.DAC_Device.DACPort                 ' Specifies the correct DAC depending on the hardware

        Call mobjParent.DAC_AnalogOutput.AnalogOutput

        ' Read the values for the Current
        Call mobjParent.DAC_DigitalOutput(mobjParent.OpenClampPattern)
        Call Pause(0.5)

        For intTissue = 1 To MAX_TISSUES
            mobjParent.DAC_AnalogInput.Channel = intTissue + MAX_TISSUES - 1                ' second bank: Current
            mobjParent.DAC_AnalogInput.Differential = False
            mobjParent.DAC_AnalogInput.Gain = mobjParent.DAC_Device.ReferenceGain
            mobjParent.DAC_AnalogInput.Unipolar = False
            dblValue = 0
            lngValue = 0

            For intSample = 1 To intNumSamples
                dblValue = dblValue + mobjParent.DAC_AnalogInput.AnalogInputVolts
                lngValue = lngValue + mobjParent.DAC_AnalogInput.AnalogInputBits
                If intTissue = 1 Then
                    'Debug.Print "Channel (I) " & intTissue - 1, mobjParent.DAC_AnalogInput.AnalogInputBits
                End If
            Next intSample

            marrReference(intTissue, 1).I = (dblValue / intNumSamples) * 1000                  ' * 1000 to convert to MicroAmps
            marrReference(intTissue, 1).IBits = lngValue / intNumSamples
        Next intTissue


        ' Read the values for the Voltage
        Call mobjParent.DAC_DigitalOutput(mobjParent.VoltageClampPattern)
        ' Pause for 1/2 second
        Call Pause(0.5)

        For intTissue = 1 To MAX_TISSUES
            mobjParent.DAC_AnalogInput.Channel = intTissue - 1                              ' First bank: Voltage
            mobjParent.DAC_AnalogInput.Differential = False
            mobjParent.DAC_AnalogInput.Gain = mobjParent.DAC_Device.ReferenceGain
            mobjParent.DAC_AnalogInput.Unipolar = False
            dblValue = 0
            lngValue = 0

            For intSample = 1 To intNumSamples
                lngValue = lngValue + mobjParent.DAC_AnalogInput.AnalogInputBits
                dblValue = dblValue + mobjParent.DAC_AnalogInput.AnalogInputVolts
                If intTissue = 1 Then
                    'Debug.Print "Channel (V) " & intTissue - 1, mobjParent.DAC_AnalogInput.AnalogInputBits
                End If
            Next intSample

            marrReference(intTissue, 1).V = (dblValue / intNumSamples) * 1000                  ' * 1000 to convert to MicroVolts
            marrReference(intTissue, 1).VBits = (lngValue / intNumSamples)
        Next intTissue

        mblnReferenced = True
    End If
End Function

Public Sub SetRefSatCurr(ByVal Index As Integer, ByVal intGain As Integer, ByVal Value As Boolean)
    marrRefSatCurr(Index, intGain) = Value
End Sub

Public Function GetRefSatCurr(ByVal Index As Integer, ByVal intGain As Integer) As Boolean
    GetRefSatCurr = marrRefSatCurr(Index, intGain)
End Function

Public Sub SetRefSatVolt(ByVal Index As Integer, ByVal intGain As Integer, ByVal Value As Boolean)
    marrRefSatVolt(Index, intGain) = Value
End Sub

Public Function GetRefSatVolt(ByVal Index As Integer, ByVal intGain As Integer) As Boolean
    GetRefSatVolt = marrRefSatVolt(Index, intGain)
End Function

Public Sub SetRefCurr(ByVal Index As Integer, ByVal intGain As Integer, ByVal Value As Double)
    marrReference(Index, intGain).I = Value
End Sub

Public Sub SetRefCurrBits(ByVal Index As Integer, ByVal intGain As Integer, ByVal Value As Long)
    marrReference(Index, intGain).IBits = Value
End Sub

Public Sub SetRefVolt(ByVal Index As Integer, ByVal intGain As Integer, ByVal Value As Double)
    marrReference(Index, intGain).V = Value
End Sub

Public Sub SetRefVoltBits(ByVal Index As Integer, ByVal intGain As Integer, ByVal Value As Long)
    marrReference(Index, intGain).VBits = Value
End Sub

Public Function ReferenceCurrent(Index As Integer, ByVal intGain As Integer) As Double
    If Index >= 1 And Index <= MAX_TISSUES Then
        ReferenceCurrent = marrReference(Index, intGain).I
    Else
        ReferenceCurrent = 0
    End If
End Function

Public Function ReferenceVoltage(Index As Integer, ByVal intGain As Integer) As Double
    If Index >= 1 And Index <= MAX_TISSUES Then
        ReferenceVoltage = marrReference(Index, intGain).V
    Else
        ReferenceVoltage = 0
    End If
End Function

Public Function ReferenceCurrentBits(Index As Integer, ByVal intGain As Integer) As Long
    If Index >= 1 And Index <= MAX_TISSUES Then
        ReferenceCurrentBits = marrReference(Index, intGain).IBits
    Else
        ReferenceCurrentBits = 0
    End If
End Function

Public Function ReferenceVoltageBits(Index As Integer, ByVal intGain As Integer) As Long
    If Index >= 1 And Index <= MAX_TISSUES Then
        ReferenceVoltageBits = marrReference(Index, intGain).VBits
    Else
        ReferenceVoltageBits = 0
    End If
End Function

Public Property Let Parent(objParent As clsMainDAC)
    Set mobjParent = objParent
End Property

Public Sub ReleaseParent()
    Set mobjParent = Nothing
End Sub

' Returns the gain value based on the index (1,2,3,4) -> (1,2,5,10)
Public Function GetGainVal(intGainIndex As Integer) As Integer
    Dim intVal As Integer


    Select Case intGainIndex
        Case 1:
            intVal = 1

        Case 2:
            intVal = 2

        Case 3:
            intVal = 5

        Case 4:
            intVal = 10

        Case Else:
            ' Error (?)
            intVal = 10
    End Select

    GetGainVal = intVal
End Function

' Reverse of previous function
Public Function GetGainIndex(intGainValue As Integer) As Integer
    Dim intIndex As Integer


    Select Case intGainValue
        Case 10:
            intIndex = 4

        Case 8:
            intIndex = 4

        Case 5:
            intIndex = 3

        Case 4:
            intIndex = 3

        Case 2:
            intIndex = 2

        Case 1:
            intIndex = 1

        Case Else:
            ' Error (?)
            intIndex = 1
    End Select

    GetGainIndex = intIndex
End Function

Private Sub Class_Initialize()
    DACAddReferenceCount "clsReference"
End Sub

Private Sub Class_Terminate()
    Erase marrReference

    DACRemoveReferenceCount "clsReference"
End Sub
